/*
 * PAC.c
 */

#include "lcd.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>


#define DIST_GET_BTN PB0
#define AREA_DEL_BTN PB1
#define AREA_SAVE_BTN PB2

static volatile unsigned int phase = 0; /* nivel logic */
static volatile unsigned int count = 0; /* numar pulsuri */
static volatile unsigned int lungime = 0;
static volatile unsigned int diagonala = 0;
static volatile unsigned int arie = 0;
static volatile unsigned int arie_totala = 0;
static unsigned int nr_mas = 0;

int8_t GET_flag = 0, DST1_flag = 0, DST2_flag = 0, DEL_flag = 0;

/* rutina de intreruperi la schimbarea de nivel logic */
ISR(PCINT0_vect)
{
	if (phase == 0) {
		/* enable counter */
		TCCR1B |= (1 << CS11);
		phase = 1;
	} else if (phase == 1) {
			/* reset and disable counter */
			TCCR1B = 0;
			count = TCNT1; /* get the count range */
			TCNT1 = 0;
			phase = 0;
	}
}

/* compute area knowing height and diagonal*/
int compute_area(int lung, int diag)
{
	double x, x2;
	x2 = pow(((double) diag), 2) - pow(((double) lung), 2);
	x = sqrt(x2);
	
	return lung*((int) x);
}


/* initializare intreruperi, display si configurare pini */
void IO_init(void)
{
	sei(); /*enable SREG*/
	
	PCICR |= (1 << PCIE0);
	PCMSK0 |= (1 << PCINT0);
	
	LCD_init();
	
	
	DDRD |= (1 << PD7);
	PORTD &= ~(1 << PD7);
	
	DDRB |= (1 << PB3);
	DDRB &= ~((1 << PB0) | (1 << PB1));
	PORTB |= (1 << PB0) | (1 << PB1);
}


/* compute and print distance on lcd */
void print_distance(void)
{
	char distBuf[16], pac[] = "PAC!", cm[] = "cm";
	
	int distance, distCopy;
	//int8_t PB0_flag = 0, PB1_flag = 0;
	
	while (1)
	{
		LCD_printAt(2, pac, 1); /* Tupac FTW !!! */
		
		/* give signal */
		PORTB |= (1 << PB3);
		_delay_us(15);
		PORTB &= ~(1 << PB3);
		
		/* compute distance */
		distance = count / (53*2);
		if (distance == 0)
			continue;
		
		/* get number of chars in distance */
		distCopy = distance;
		
		LCD_writeInstr(LCD_INSTR_clearDisplay);			
		if (distance>0 && distance<=400)
		{
			itoa(distance, distBuf, 10);
			
			LCD_printAt(2, pac, 1); 
			LCD_printAt(9, distBuf, 1);	
			LCD_printAt(13, cm, 1);			
		}
		else
		{
			LCD_printAt(2, pac, 1); 
			LCD_printAt(7, ">MAXRange", 1);
		}
		
		if(!(PINB & (1 << DIST_GET_BTN)) && !GET_flag) {
			 GET_flag = 1;
            //PB0 apasat
			PORTD |= (1 << PD7);			
			nr_mas++;
			if (nr_mas == 1) {
				lungime = distance;
				DST1_flag = 1;
				break;
			}
			if (nr_mas == 2) {
				diagonala = distance;
				arie = compute_area(lungime, diagonala);
				DST2_flag = 1;
				break;
			}
		}
		
		
		/*if((PINB & (1 << DIST_GET_BTN)) && GET_flag) {
            GET_flag = 0;
            //PB0 lasat;
			PORTD &= ~(1 << PD7);
        }*/
		
		_delay_ms(5);
		
	}
	
}

/* redare ton buzzer */
void speaker_morse(int tip)
{
    int delay = 1;
    int i;
    DDRB |= (1 << PB4);     // PD5 trebuie sa fie output

    for (i = 0; i < tip * 50 / delay; i++)
    {
        PORTB ^= (1 << PB4); // alternam starea iesirii
        _delay_ms(1);
    }
}

int main()
{
	char dBuf[5], aBuf[5], c_m[] = "cm", m_2[] = "m^2";
	
	IO_init();

	while (1)
	{
		print_distance();
		
		if (DST1_flag == 1) {
			itoa(lungime, dBuf, 10);
			LCD_printAt(9, dBuf, 2);			
			LCD_printAt(13, c_m, 2);
			
			DST1_flag = 0;
			GET_flag = 0;
			PORTD &= ~(1 << PD7);
		}
		
		if (DST2_flag == 1) {
			itoa(diagonala, dBuf, 10);
			LCD_printAt(9, dBuf, 2);			
			LCD_printAt(13, c_m, 2);
			
			arie = compute_area(lungime,diagonala);
			arie_totala += arie;
			
			
			itoa(arie_totala/10000, aBuf, 10);
			LCD_printAt(2, aBuf, 2);			
			LCD_printAt(6, m_2, 2);				
			
			nr_mas = 0;
			lungime = 0;
			diagonala = 0;
			arie = 0;
			
			DST2_flag = 0;
			GET_flag = 0;
			PORTD &= ~(1 << PD7);
		}
	}
	
    return 0;
}

